# MAIN

if test -n "$ZSH_VERSION" && (emulate sh) > /dev/null 2>&1 ; then
    emulate sh
    NULLCMD=:
  # zsh still fscks up backslash-escaped chracters in for i in $var constructs,
  # while sh/ash/bsh/ksh/bash do it right. how do i fix that?
elif test -n "$BASH_VERSION" && (set -o posix) > /dev/null 2>&1 ; then
    set -o posix
fi

# ash/bsh does not seem to set a correct IFS. no harm in setting it for
# every shell
IFS=" 	
"

# turn off locale crap
LC_ALL=C
export LC_ALL

# check for BSD or SYSV echo
case `echo -n` in
    -n) _echo_n=     _echo_c='\c' ;;
    *)  _echo_n=-n   _echo_c= ;;
esac

# check if backslash-escaped control characters need -e
case `echo "\r"` in
    "\r") _echo_e=-e ;;
    *)    _echo_e= ;;
esac

# does not preserve backslash-escaped control characters
# maximum of six fields (can be expanded ofcourse)
cut_fast() {
    SAVIFS="$IFS"
    IFS="$1"; read aa1 aa2 aa3 aa4 aa5 aa6; eval echo \"\$aa$2\";
    IFS="$SAVEIFS"
}

# start with empty list
_loaded_modules=""

# function every module needs to call ro register itself
conf_module() {
    _loaded_modules="$_loaded_modules ;$1;"
}

# register ourselves
conf_module main

# debug function
conf_print_loaded_modules() {
    echo "loaded modules: $_loaded_modules"
}

# check if module is loaded
conf_check_module_loaded() {
    case "$_loaded_modules" in
        *\;$1\;*)   test 1 -eq 1 ;;
        *)          test 1 -eq 0 ;;
    esac
}

# generic function to load modules
conf_load_module() {
    for i in $@ ; do
        conf_check_module_loaded "$i" && return

        if test -f "conf/conf_$i.sh" ; then
            . "conf/conf_$i.sh"
        elif test -f "conf/misc/conf_$i.sh" ; then
            . "conf/misc/conf_$i.sh"
        elif test -f "conf/cheaders/conf_$i.sh" ; then
            . "conf/cheaders/conf_$i.sh"
        elif test -f "conf/clibs/conf_$i.sh" ; then
            . "conf/clibs/conf_$i.sh"
        elif test -f "conf/audio/conf_$i.sh" ; then
            . "conf/audio/conf_$i.sh"
        elif test -f "conf/video/conf_$i.sh" ; then
            . "conf/video/conf_$i.sh"
        elif test -f "conf/io/conf_$i.sh" ; then
            . "conf/io/conf_$i.sh"
        else
            die "Configure is trying to load module: $i which does not exist!"
        fi
    done
}

# start with empty list of tests done
_tests_done=""

# debug function
conf_print_tests_done() {
    echo "tests done: $_tests_done"
}

# mark a test as done
conf_mark_test_done() {
    _tests_done="$_tests_done ;$1;"
}

# check if a test is done
conf_check_test_done() {
    case "$_tests_done" in
        *\;$1\;*)   test 1 -eq 1 ;;
        *)          test 1 -eq 0 ;;
    esac
}

# check dependency and, if necessary, load module and do test
conf_depends_on() {
    if conf_check_test_done "$1" ; then
        eval test -n \"\$_$1\"
        return
    fi

    conf_check_module_loaded "$1" || conf_load_module "$1"

    eval conf_test_$1 \"$2\" \"$3\" \"$4\" \"$5\" \"$6\" \"$7\"
}

# MAYBE make this call conf_<modulename>_cleanup or something
conf_cleanup() {
    test -n "$_cc_run_i" && rm -f "$_cc_run_i"
    test -n "$_cc_run_o" && rm -f "$_cc_run_o"
    test -n "$_cxx_run_i" && rm -f "$_cxx_run_i"
    test -n "$_cxx_run_o" && rm -f "$_cxx_run_o"
}

# call die instead of exit elsewhere
die() {
    echo $@ 1>&2
    conf_cleanup
    exit 1
}

# -----------------------------------------------------------------------------

# load configure defaults
conf_load_module defaults

# command line option parsing
conf_load_module options

# defines and undefines
conf_load_module defines

# misc. util functions without any deps
conf_load_module util

# default sections and options can be initialized with init functions
conf_load_module init
