conf_module makefile

_maktargets=

# mak_target name type
#
# type is one of:
#       static_library | archive | a
#       shared_library | so
#       objects | o
#       application | app

mak_target() {
    case $2 in
        static_library|archive|a)   eval _mak_${1}_type=a ;;
        shared_library|so)          eval _mak_${1}_type=so ;;
        objects|o)                  eval _mak_${1}_type=o ;;
        application|app)            eval _mak_${1}_type=app ;;
        *)
            die "unknown Makefile target type! error in configure!"
            ;;
    esac

    _maktargets="$_maktargets $1 "
}

mak_check_target() {
    case "$_maktargets" in
        *\ $1\ *) ;;
        *)
            die "referencing unknown target ($1). error in configure!"
            ;;
    esac
}

mak_target_inputs() {
    mak_check_target $1 && eval _mak_${1}_inputs=\"$2\"
}

mak_target_cppflags() {
    mak_check_target $1 && eval _mak_${1}_cppflags=\"$2\"
}

mak_target_asflags() {
    mak_check_target $1 && eval _mak_${1}_asflags=\"$2\"
}

mak_output_makefile() {
    conf_depends_on make || die "cannot build without make!"

    echo "${_echo_cyan}writing $1$_echo_normal"

    all_outputs=""
    mak_depend_deps=""

    cat >$1.new <<EOF
# ---------- Generated by configure ----------
# No default rules for suffixes
.SUFFIXES:

include config.mak

EOF

# all: rule --> all application targets

    for maktarget in $_maktargets ; do
        eval tmp=\$_mak_${maktarget}_type
        case $tmp in
            o|a|so) ;;
            app)    all_apps="$maktarget $all_apps" ;;
        esac
    done

    echo "all: $all_apps" >> $1.new
    echo "" >> $1.new

# include .depend
    cat >>$1.new <<EOF
ifneq (\$(wildcard .depend),)
include .depend
endif

EOF

# output all targets

    for maktarget in $_maktargets ; do
        eval tmp=\$_mak_${maktarget}_type
        case $tmp in
            o)      mak_output_objects $1.new $maktarget ;;
            a)      mak_output_archive $1.new $maktarget ;;
            so)     ;;
            app)    mak_output_application $1.new $maktarget ;;
        esac
    done

# output clean/distclean

    cat >>$1.new <<EOF
.PHONY: clean distclean
clean:
	rm -f $all_outputs
distclean: clean
	rm -f .depend

EOF

# output install/uninstall

    # FIXME: not only install apps and shared libraries but also have
    # configure define manual pages, info files, default rc files, images,
    # etc...

# output dep/depend/.depend
    conf_depends_on makedepend

    test -z "$_makedepend" && die "need makedepend!"

    cat >>$1.new <<EOF
.PHONY: dep depends
dep: .depend
depend: .depend
.depend:
	rm -f .depend
	\$(MAKE) -f $1 $mak_depend_deps

EOF

    mak_local_makefile >>$1.new

    diff $1 $1.new 2>/dev/null 1>&2

    if test "$?" = "0" ; then
        rm -f $1.new
    else
        mv $1 $1.old 2>/dev/null
        mv $1.new $1
    fi

    echo "${_echo_cyan}$1 done$_echo_normal"
}

mak_output_objects() {
    conf_depends_on sed

    # FIXME: and maybe other mak_output* functions. might also use awk or
    # perhaps something else to stip off the .extension
    # bash/ksh/zsh/ash (at least the version(s) i use, support ${file%%.ext}
    # but sunos 5.9 /bin/sh does not!
    test -z "$_sed" && die "I really need sed!"

    eval inputs=\$_mak_${2}_inputs
    outputs=""

    eval echo ${2}_cppflags = \$_mak_${2}_cppflags >>$1
    eval echo ${2}_asflags = \$_mak_${2}_asflags >>$1
    echo "" >>$1

    all_inputs=""

    for inp in $inputs ; do
        case $inp in
            *.c|*.cpp|*.cxx|*.cc|*.s|*.S) ;;
            *)
                echo "unknown filetype ($inp) in $2 inputs!" >&2
                die "error in configure!"
                ;;
        esac

        object=`echo $inp | $_sed 's/\.[^\.]*$//'`.o

        echo "$object: $inp" >>$1

        case $inp in
            *.c)
                conf_depends_on cc
                all_inputs="$inp $all_inputs"
                echo "	\$(CC) -c \$(CFLAGS) \$(CPPFLAGS) \$(${2}_cppflags) -o \$@ \$<" >>$1
                ;;
            *.cpp|*.cxx|*.cc)
                conf_depends_on cxx
                all_inputs="$inp $all_inputs"
                echo "	\$(CXX) -c \$(CXXFLAGS) \$(CPPFLAGS) \$(${2}_cppflags) -o \$@ \$<" >>$1
                ;;
            *.s|*.S)
                conf_depends_on as
                echo "	\$(AS) \$(ASFLAGS) \$(${2}_asflags) -o \$@ \$<" >>$1
                ;;
            *)
                echo ""
        esac

        outputs="$object $outputs"
    done
    echo "" >>$1

# FIXME: gcc -MM sucks because it strips off the directoryname of the target.
# if makedepend is too obscure, stuff can be "fixed" with a script and the -MT
# option of gcc/icc/etc.
# Aaargh! makedepend sucks too. for example, if libaf/af_sweep.c includes
# "config.h" offcourse libaf/config.h is meant. All compilers know that, but
# makedepend does not look in the directory of the sourcefile first, but
# in the directory from which makedepend was called!! So it names ./config.h
# as a dependency instead of ./libaf/config.h ...... *sigh*
# Probably fix this with -MM -MT stuff or a cpp/grep/sed/whatever pipeline

    cat >>$1 <<EOF
${2}_depend:
	\$(MAKEDEPEND) -f- -- \$(CPPFLAGS) -Y \$(${2}_cppflags) -- $all_inputs >> .depend 2>/dev/null

EOF

    mak_depend_deps="${2}_depend $mak_depend_deps"

    all_outputs="$outputs $all_outputs"

    eval _mak_${2}_outputs=\"$outputs\"
}

mak_target_objects() {
    mak_check_target $1 && eval _mak_${1}_objects=\"$2\"
}

mak_target_arflags() {
    mak_check_target $1 && eval _mak_${1}_arflags=\"$2\"
}

mak_target_output() {
    mak_check_target $1 && eval _mak_${1}_output=\"$2\"
}

mak_output_archive() {
    conf_depends_on ar || die "need ar for static libraries!"
    conf_depends_on ranlib


    eval echo ${2}_arflags = \$_mak_${2}_arflags >>$1
    echo "" >>$1

    eval objects=\$_mak_${2}_objects
    inputs=""
    for obj in $objects ; do
        eval type=\$_mak_${obj}_type
        test "$type" != "o" && \
            echo "$2 references non-objects target $obj" && \
            die "error in configure!"
        eval tmp=\"\$_mak_${obj}_outputs\"
        inputs="$tmp $inputs"
    done

    eval output=\$_mak_${2}_output

    all_outputs="$output $all_outputs"

    echo "$output: $inputs" >>$1
    echo "	\$(AR) \$($2_arflags) r \$@ \$?" >>$1
    echo "	\$(RANLIB) \$@" >>$1

    echo "" >>$1
}

mak_target_archives() {
    mak_check_target $1 && eval _mak_${1}_archives=\"$2\"
}

mak_target_ldflags() {
    mak_check_target $1 && eval _mak_${1}_ldflags=\"$2\"
}

mak_output_application() {
    conf_depends_on cc      # default link with cc. user can override by
                            # having a check for ld in configure

    eval echo ${2}_ldflags = \$_mak_${2}_ldflags >>$1
    echo "" >>$1

    eval objects=\$_mak_${2}_objects
    inputs=""
    for obj in $objects ; do
        eval type=\$_mak_${obj}_type
        test "$type" != "o" && \
            echo "$2 references non-objects target $obj" && \
            die "error in configure!"
        eval tmp=\"\$_mak_${obj}_outputs\"
        inputs="$tmp $inputs"
    done

    eval archives=\$_mak_${2}_archives
    for archive in $archives ; do
        eval type=\$_mak_${archive}_type
        test "$type" != "a" && \
            echo "$2 references non-archive target $archive" && \
            die "error in configure!"
        eval tmp=\"\$_mak_${archive}_output\"
        inputs="$tmp $inputs"
    done

    all_outputs="$2 $all_outputs"

    echo "$2: $inputs" >>$1
    echo "	\$(LD) -o \$@ \$^ \$($2_ldflags)" >>$1

    echo "" >>$1
}
