conf_module options

command_line_sections=

# conf_add_sections [section[|section[...]]]
#
# section = name;description;
#
# add names and descriptions of command line options sections to the list

conf_add_sections() {
    SAVEIFS="$IFS"

    if test -z "$command_line_sections" ; then
        command_line_sections="$@"
    else
        command_line_sections="$command_line_sections\|$@"
    fi

    IFS="|"; for i in "$1" ; do
        IFS=";"; read section description <<EOF
$i
EOF
        eval command_line_section_$section=""
     done

     IFS="$SAVEIFS"
}

# conf_add_options section [option[|option[...]]]
#
# option = "name;variable;description;"
#
# add command line options to its sectionlist

conf_add_options() {
    SAVEIFS="$IFS"

    eval tmp=\$command_line_section_$1
    if test -z "$tmp" ; then
        IFS=" "; eval command_line_section_$1=\'$2\'
    else
        IFS=" "; eval command_line_section_$1=\"\$command_line_section_$1\|\"\'$2\'
    fi

    IFS="$SAVEIFS"
}

# print an option
#
# conf_print_option option description

conf_print_option() {
# some hosts overwrite previously printed characters if \r is used
# don't rely on printf or awk is available

    case "$1" in
    ?) echo $_echo_e "$1                       $2" ;;
    ??) echo $_echo_e "$1                      $2" ;;
    ???) echo $_echo_e "$1                     $2" ;;
    ????) echo $_echo_e "$1                    $2" ;;
    ?????) echo $_echo_e "$1                   $2" ;;
    ??????) echo $_echo_e "$1                  $2" ;;
    ???????) echo $_echo_e "$1                 $2" ;;
    ????????) echo $_echo_e "$1                $2" ;;
    ?????????) echo $_echo_e "$1               $2" ;;
    ??????????) echo $_echo_e "$1              $2" ;;
    ???????????) echo $_echo_e "$1             $2" ;;
    ????????????) echo $_echo_e "$1            $2" ;;
    ?????????????) echo $_echo_e "$1           $2" ;;
    ??????????????) echo $_echo_e "$1          $2" ;;
    ???????????????) echo $_echo_e "$1         $2" ;;
    ????????????????) echo $_echo_e "$1        $2" ;;
    ?????????????????) echo $_echo_e "$1       $2" ;;
    ??????????????????) echo $_echo_e "$1      $2" ;;
    ???????????????????) echo $_echo_e "$1     $2" ;;
    ????????????????????) echo $_echo_e "$1    $2" ;;
    ?????????????????????) echo $_echo_e "$1   $2" ;;
    ??????????????????????) echo $_echo_e "$1  $2" ;;
    *) echo $_echo_e "$1 $2" ;;
    esac
}

# conf_print_options
#
# print a list of all command line options with their description, grouped
# by section. omit options whose description matches xxx in order to be able
# to have --enable-*, --disable-* and --force-* options for the same variable
# but print only one

conf_print_options() {
    SAVEIFS="$IFS"

    echo
    echo "Usage: ./configure [OPTION]..."
    echo

    IFS="|"; for i in $command_line_sections ; do
        IFS=";"; read section description <<EOF
$i
EOF
        IFS="$SAVEIFS;"; echo $_echo_e $description
        echo

        eval tmp=\$command_line_section_$section
        IFS="|"; for j in $tmp ; do
            IFS=";"; read fulloption variable optiondescription xxx <<EOF
$j
EOF
            IFS="$SAVEIFS;"
            if test "$optiondescription" != "xxx" ; then
                conf_print_option "$fulloption" "$optiondescription"
            fi
        done

        echo
    done

    IFS="$SAVEIFS"
}

# conf_parse_options [option] ...
#
# parse command line options

conf_parse_options() {
    SAVEIFS="$IFS"

# return if no options are given. avoids parser being build. helps to
# to lower startup time on very slow machines
    if test "$#" = "0" ; then
        return
    fi

    for i do
        if test "$i" = "-h" ; then
            conf_print_options
            exit
        elif test "$i" = "--help" ; then
            conf_print_options
            exit
        fi
    done

# build parser once instead of checking all sections for every option in
# a loop with if then else fi (which is terribly slow on very old machines)

    matchoption=`{
        echo $_echo_n "case \\$ioption in " $_echo_c

        IFS="|"; for j in $command_line_sections ; do
            IFS=";"; read section description <<EOF
$j
EOF
            eval tmp=\\$command_line_section_$section

            IFS="|"; for k in $tmp ; do
                IFS=";"; read fulloption variable optiondescription xxx <<EOF
$k
EOF
                option=\` echo $fulloption | cut_fast '=' 1 \`
                IFS="$SAVEIFS"
                echo $_echo_n "$option) fulloption=$fulloption; variable=$variable;; " $_echo_c

            done
        done
        echo "*) die Unknown command line option: \\$ioption ;;"
        echo "esac"
    }`

    IFS="$SAVEIFS"

    parser=`{
        echo "\
            for i do \
                case \\$i in \
                    [A-Z]*=*) \
                        conf_setenv \\$i ; \
                        continue; \
                        ;; \
                esac; \
                ioption=\\\` echo \\$i | cut_fast '=' 1 \\\` ; \
                ivalue=\\\` echo \"\\$i=\" | cut_fast '=' 2 \\\` ; \
                $matchoption ; \
                option=\\\` echo \\$fulloption | cut_fast '=' 1 \\\` ; \
                if test \"\\$ioption\" = \"\\$option\" ; then \
                    value=\\\` echo \"\\$fulloption=\" | cut_fast '=' 2 \\\` ; \
                    if test -n \"\\$value\" ; then \
                        eval \\$variable=\\$ivalue ; \
                    else if test -n \"\\$ivalue\" ; then \
                        die \"Command line option \\$option does not take an argument\" ; \
                    fi ; \
                    action=\\\` echo \"\\$i\"-yes | cut_fast '-' 3 \\\` ; \
                    eval \\$variable=\\$action ; \
                fi ; \
            fi ; \
        done"
    }`

# run the parser
    eval $parser

    IFS="$SAVEIFS"
}

conf_setenv() {
    case "`echo $1 | cut_fast '=' 1`" in
        OS|OS_RELEASE|ARCH|CPU|CPU_FEATURES|CC|CXX)
            ;;
        *)
            echo "You specified $1. Setting this variable does not affect"
            echo "the operation of 'configure'. Maybe you made a typo?"
            die
            ;;
    esac
    eval $1
}
