conf_module arch

_arch=

conf_add_options "expert" \
    "--arch=ARCH;ARCH;specify the target architecture;"

conf_test_arch() {
    conf_depends_on tolower
    conf_depends_on target

    if test -n "$ARCH" ; then
        _arch=`echo "$ARCH" | $_tolower`
        echocheck "architecture"
        _arch=`conf_known_archs "$_arch"`
        echores "$_arch"
        conf_mark_test_done arch
        return
    fi

    conf_depends_on uname

    echocheck "architecture"

    if test -n "$_uname" ; then
        tmp=`$_uname -p 2>&1 | $_tolower`
        case "$tmp" in
        i386|sparc|ppc|alpha|arm|mips|vax|powerpc|powerpc64|sparc64|x86_64)
            _arch=$tmp
            ;;
        *)  # try uname -m if uname -p does not return a valid or known type
            tmp=`$_uname -m 2>&1 | $_tolower`
            _arch=`conf_known_archs "$tmp"`
            ;;
        esac
    fi

    if test -z "$_arch" ; then
        # just call cc and see if it knows -dumpmachine
        tmp=`cc -dumpmachine 2>&1 | $_tolower | cut_fast '-' 1`
        _arch=`conf_known_archs "$tmp"`
    fi

    test -z "$_arch" && _arch=unknown

    conf_add_define ARCH_$_arch

    echores "$_arch"
    conf_mark_test_done arch
    test "$_arch" != "unknown"
}

conf_known_archs() {
    case "$1" in
        k5|k6|k6_2|k6_3|k6-2|k6-3)  echo i386 ;;
        nsr-[gklnptvw])     echo nsr ;;
        cray*[a-z]90)   echo $1 ;;
        night_hawk)     echo powerpc ;;
        9000/[34]?)     echo m68k ;;
        x86|x86pc)  echo i386 ;;
        powerpcle)  echo powerpcle ;;
        powerpc64)  echo powerpc64 ;;
        i586-i686)  echo i386 ;;
        i586_i686)  echo i386 ;;
        i[3-9]86*)  echo i386 ;;
        cray*y-mp)  echo ymp ;;
        sempron*)   echo i386 ;;
        r[34]000)   echo mips ;;
        pyramid*)   echo pyramid ;;
        pentium*)   echo i386 ;;
        parisc64)   echo parisc64 ;;
        cray*t3?)   echo alpha ;;
        cray*sv1)   echo sv1 ;;
        celeron*)   echo i386 ;;
        tsunami)    echo sparc ;;
        sparc64)    echo sparc64 ;;
        sm[be]s)    echo mips ;;
        powerpc)    echo powerpc ;;
        parisc*)    echo hppa ;;
        paragon)    echo i860 ;;
        mvmeppc)    echo powerpc ;;
        mvme68k)    echo m68k ;;
        mc68030)    echo m68k ;;
        f30[01])    echo $1 ;;
        cray*ts)    echo t90 ;;
        athlon*)    echo i386 ;;
        alpha\ *)   echo alpha ;;
        zaurus)     echo arm ;;
        x86_64)     echo x86_64 ;;
        wgrisc)     echo mipsel ;;
        sr8000)     echo hppa ;;
        sr2?01)     echo hppa ;;
        sparc*)     echo sparc ;;
        rs6000)     echo rs6000 ;;
        rs6000)     echo rs6000 ;;
        milan*)     echo m68k ;;
        macppc)     echo powerpc ;;
        mac68k)     echo m68k ;;
        m680??)     echo $1 ;;
        hades*)     echo m68k ;;
        duron*)     echo i386 ;;
        bs2000)     echo bs2000 ;;
        aviion)     echo m88k ;;
        atari*)     echo m68k ;;
        armeb*)     echo armeb ;;
        amiga*)     echo m68k ;;
        9000/*)     echo hppa ;;
        xkl-1)  echo pdp10 ;;
        xd88*)  echo m88k ;;
        sun4*)  echo sparc ;;
        sun3*)  echo m68k ;;
        sh3el)  echo shl ;;
        sh3eb)  echo sh ;;
        s390x)  echo s390x ;;
        s390x)  echo s390x ;;
        risc*)  echo mips ;;
        prep*)  echo powerpcle ;;
        ppc64)  echo powerpc64 ;;
        pdp10)  echo pdp10 ;;
        ns32k)  echo ns32k ;;
        nile*)  echo pyramid ;;
        news*)  echo mips ;;
        mips*)  echo mips ;;
        mini*)  echo m68010 ;;
        intel)  echo i386 ;;
        ibmrt)  echo romp ;;
        hppa*)  echo hppa ;;
        hp300)  echo m68k ;;
        bemac)  echo powerpc ;;
        bebox)  echo powerpc ;;
        aushp)  echo sparc ;;
        amd64)  echo x86_64 ;;
        alpha)  echo alpha ;;
        21064)  echo alpha ;;
        vax*)   echo vax ;;
        sx-5)   echo sx5 ;;
        sx-4)   echo sx4 ;;
        s390)   echo s390 ;;
        s390)   echo s390 ;;
        pmax)   echo mipsel ;;
        mips)   echo mips ;;
        m88k)   echo m88k ;;
        m88*)   echo m88k ;;
        m68k)   echo m68k ;;
        m68*)   echo m68k ;;
        ks10)   echo pdp10 ;;
        ia64)   echo ia64 ;;
        i860)   echo i860 ;;
        i370)   echo i370 ;;
        cats)   echo arm ;;
        c38*)   echo c38 ;;
        c34*)   echo c34 ;;
        BePC)   echo i386 ;;
        arm*)   echo arm ;;
        2020)   echo clipper ;;
        sgi)    echo mipseb ;;
        rm*)    echo mips ;;
        ppc)    echo powerpc ;;
        c4*)    echo c4 ;;
        c2*)    echo c2 ;;
        c1*)    echo c1 ;;
        arc)    echo mipsel ;;
        i*)     echo i386 ;;
        *falcon*)   echo m68k ;;
        ????????)   echo romp ;;
        *)          echo ;; 
    esac
}
