conf_module cc

_cc=
_cc_vendor=
_cc_vendor_long=
_cc_version=
_cc_version_major=
_cc_version_minor=
_cc_version_subminor=

conf_add_options "expert" \
    "--cc=CC;CC;specify C compiler;"

conf_test_cc() {
    conf_depends_on program _cc "C compiler" "$CC cc gcc icc tcc" \
                                                        conf_test_cc_validate
    conf_mark_test_done cc
    test -n "$_cc" || return

    echocheck "C compiler vendor"
    conf_cc_vendor
    if test -z "$_cc_vendor_long" ; then
        echores "$_cc_vendor"
    else
        _res_comment="$_cc_vendor"
        echores "$_cc_vendor_long"
    fi

    conf_cc_version

    test -n "$_cc"
}

conf_test_cc_validate() {
    $1 -v && return
    $1 -version && return
    $1 --version && return
    $1 -flags && return
}

conf_cc_vendor() {
    tmp=`$_cc -v 2>&1`
    case "$tmp" in
        tcc\ version*)
            _cc_vendor=tcc
            _cc_vendor_long="Tiny C Compiler by Fabrice Bellard"
            return
            ;;
    esac

    tmp=`$_cc --help 2>&1`
    tmp=`echo $tmp`
    case "$tmp" in
        Intel*)
            _cc_vendor=icc
            _cc_vendor_long="Intel(R) C/C++ Compiler"
            return
            ;;
    esac

    tmp=`$_cc --version 2>&1`
    case "$tmp" in
        gcc*|*GCC*|2.95*)
            _cc_vendor=gnu
            _cc_vendor_long="GNU C Compiler"
            return
            ;;
    esac

    tmp=`$_cc -flags 2>&1`
    case "$tmp" in
        -#*-###*)
            _cc_vendor=suncc
            _cc_vendor_long="SUNWspro C Compiler"
            return
            ;;
    esac

    _cc_vendor=unknown
}

conf_cc_version() {
    case "$_cc_vendor" in
        tcc)
            echocheck "C compiler version"
            _cc_version=`$_cc -v 2>&1 | cut_fast ' ' 3`
            ;;
        icc)
            echocheck "C compiler version"
            _cc_version=`$_cc -v 2>&1 | cut_fast ' ' 2`
            ;;
        gnu)
            echocheck "C compiler version"
            _cc_version=`$_cc -dumpversion 2>&1`
            ;;
        suncc)
            conf_compile_tool_cc_version
            ;;
        unknown)
            ;;
    esac
    if test "$_cc_vendor" != "unknown" ; then
        _cc_version_major=`echo $_cc_version | cut_fast '.' 1`
        _cc_version_minor=`echo $_cc_version | cut_fast '.' 2`
        _cc_version_subminor=`echo $_cc_version | cut_fast '.' 3`
        echores "$_cc_version"
    fi
}

conf_compile_tool_cc_version() {
    conf_depends_on tempdir

    if test -z "$_tempdir" ; then
        echocheck "C compiler version"
        return
    fi

    echocheckreal "compiling cc-version helper utility"

    test -r "conf/tools/cc-version-$_cc_vendor.c" || {
        echores "not available for $_cc_vendor"
        return; }

    { $_cc -o "$_tempdir"/cc-version-$_cc_vendor conf/tools/cc-version-$_cc_vendor.c ; } 2>/dev/null 1>&2

    if test $? -eq 0 ; then
        _res_comment="using $_tempdir/cc-version-$_cc_vendor"
        echores "done"
        echocheck "C compiler version"
        _cc_version=`$_tempdir/cc-version-$_cc_vendor 2>/dev/null`
    else
        echores "failed to compile"
        echocheck "C compiler version"
    fi
}
