conf_module cc_tune

_cc_tune=

conf_test_cc_tune() {
    conf_depends_on cc
    conf_depends_on cpu
    conf_depends_on tempdir

    echocheck "$_cc CPU specific optimization"

    case "$_cc_vendor" in
    gnu|tcc)
        i=`conf_try_cc_gnu -mtune= $_cpu`
        test -z "$i" && i=`conf_try_cc_gnu -mcpu= $_cpu`
        test -n "$i" && _cc_tune="$i $_cc_tune"
        i=`conf_try_cc_gnu -march= $_cpu`
        test -n "$i" && _cc_tune="$i $_cc_tune"
        case "$_cpu_features" in
        *altivec*)
            i=`conf_try_cc_gnu "-D__APPLE_ALTIVEC__ " "-faltivec"`
            test -z "$i" && i=`conf_try_cc_gnu "-maltivec " "-mabi=altivec"`
            test -n "$i" && _cc_tune="$i $_cc_tune"
            ;;
        esac
        ;;
    icc)
        i=`conf_try_cc_icc -mcpu= $_cpu`
        test -n "$i" && _cc_tune="$i $_cc_tune"
        i=`conf_try_cc_icc -march= $_cpu`
        test -n "$i" && _cc_tune="$i $_cc_tune"
        ;;
    suncc)
        i=`conf_try_cc_suncc -xarch= $_cpu`
        test -n "$i" && _cc_tune="$i $_cc_tune"
        ;;
    *)
        ;;
    esac

    if test -n "$_cc_tune" ; then
        echores "$_cc_tune"
    else
        echores "none"
    fi

    conf_mark_test_done cc_tune
}

conf_try_cc_gnu() {
    tmp=$2

# try -moption=cpu and fall back to lesser/earlier cpu's if it fails

    while test 1 -eq 1 ; do
        conf_try_cc "$1$tmp" && break
        case "$tmp" in
            k8)           tmp=opteron ;;
            opteron)      tmp=athlon64 ;;
            athlon64)     tmp=athlon-fx ;;
            athlon-fx)    tmp=athlon-xp ;;
            athlon-xp)    tmp=athlon-4 ;;
            athlon-4)     tmp=athlon-tbird ;;
            athlon-tbird) tmp=athlon ;;
            athlon)       tmp=k6-3 ;;
            k6-3)         tmp=k6-2 ;;
            k6-2)         tmp=k6 ;;
            k6)           tmp=i586 ;;
            c3)           tmp=i586 ;;
            nocona)       tmp=prescott ;;
            prescott)     tmp=pentium4m ;;
            pentium4m)    tmp=pentium4 ;;
            pentium4)     tmp=pentium-m ;;
            pentium-m)    tmp=pentium3m ;;
            pentium3m)    tmp=pentium3 ;;
            pentium3)     tmp=pentium2 ;;
            pentium2)     tmp=i686 ;;
            i686)         tmp=pentium-mmx ;;
            pentium-mmx)  tmp=i586 ;;
            winchip-c6)   tmp=winchip2 ;;
            winchip2)     tmp=i586 ;;
            i586)         tmp=i486 ;;
            i486)         tmp=i386 ;;
            arm*)         tmp=arm ;;    # FIXME: proper cascade for arm cpu
            21264a)       tmp=ev67 ;;   # Alpha
            ev67)         tmp=ev6 ;;
            21264)        tmp=ev6 ;;
            ev6)          tmp=pca56 ;;
            21164pc)      tmp=pca56 ;;
            pca56)        tmp=ev56 ;;
            21164a)       tmp=ev56 ;;
            ev56)         tmp=ev5 ;;
            21164)        tmp=ev5 ;;
            ev5)          tmp=ev4 ;;
            21064)        tmp=ev4 ;;
            ev45)         tmp=ev4 ;;
                                        # FIXME: proper cascade for powerpc
            ultrasparc3)  tmp=ultrasparc ;;     # sparc
            ultrasparc)   tmp=v9 ;;
            v9)           tmp=sparclet ;;
            tsc701)       tmp=sparclet ;;
            sparclet)     tmp=sparclite ;;
            f934)         tmp=sparclite ;;
            f930)         tmp=sparclite ;;
            sparclite)    tmp=supersparc ;;
            supersparc)   tmp=v8 ;;
            v8)           tmp=cypress ;;
            cypress)      tmp=v7 ;;
            i386|*)       tmp=          # fallthrough
                          test 1 -eq 0
                          break ;;
        esac
    done

    test -n "$tmp" && echo "$1$tmp"
}

conf_try_cc_icc() {
    tmp=$2

    while test 1 -eq 1 ; do
        conf_try_cc $1$tmp && break
        case "$tmp" in
            k8)           tmp=pentiumpro ;;  #FIXME: or pentium4 ??
            opteron)      tmp=pentiumpro ;;
            athlon64)     tmp=pentiumpro ;;
            athlon-fx)    tmp=pentiumpro ;;
            athlon-xp)    tmp=pentiumpro ;;
            athlon-4)     tmp=pentiumpro ;;
            athlon-tbird) tmp=pentiumpro ;;
            athlon)       tmp=pentiumpro ;;
            k6-3)         tmp=pentium ;;
            k6-2)         tmp=pentium ;;
            k6)           tmp=pentium ;;
            c3)           tmp=pentium ;;
            nocona)       tmp=pentium4 ;;
            prescott)     tmp=pentium4 ;;
            pentium4m)    tmp=pentium4 ;;
            pentium4)     tmp=pentiumpro ;;
            pentium-m)    tmp=pentiumpro ;;
            pentium3m)    tmp=pentiumpro ;;
            pentium3)     tmp=pentiumpro ;;
            pentium2)     tmp=pentiumpro ;;
            pentiumpro)   tmp=i586 ;;
            i686)         tmp=i586 ;;
            pentium-mmx)  tmp=i586 ;;
            winchip-c6)   tmp=i586 ;;
            winchip2)     tmp=i586 ;;
            i586)         tmp=i486 ;;
            i486)         tmp=i386 ;;
            i386|*)       tmp=          # fallthrough
                          test 1 -eq 0
                          break ;;
        esac
    done

    test -n "$tmp" && echo "$1$tmp"
}

conf_try_cc_suncc() {
    tmp=$2

    while test 1 -eq 1 ; do
        echo "int main(void) { return 0; }" > $_tempdir/conf-test-$$.c
        k=`$_cc $1$tmp -o /dev/null $_tempdir/conf-test-$$.c 2>&1 `
        case $k in
            *illegal\ option*) ;;
            *)  break ;;
        esac
        case "$tmp" in
            v9b)        tmp=v9a ;;
            v9a)        tmp=v9 ;;
            v9)         tmp=v8plusb ;;
            v8plusb)    tmp=v8plusa ;;
            v8plusa)    tmp=v8plus ;;
            v8plus)     tmp=v8 ;;
            v8)         tmp=v8a ;;
            v8a)        tmp=v7 ;;
            v7)         tmp=native ;;
            native)     tmp=generic ;;
            generic)    tmp=
                        test 1 -eq 0
                        break ;;
            *)          tmp=native ;;
        esac
    done

    test -n "$tmp" && echo "$1$tmp"
}

conf_try_cc() {
    echo "int main(void) { return 0; }" > $_tempdir/conf-test-$$.c
    $_cc $1 -o /dev/null $_tempdir/conf-test-$$.c 2>/dev/null 1>&2
}
