conf_module cpu

_cpu=
_cpu_vendor=unknown
_cpu_family=unknown
_cpu_model=unknown
_cpu_modelname=unknown
_cpu_stepping=unknown
_cpu_features=none

conf_add_options "expert" \
    "--cpu=CPU;CPU;specify the target CPU;
    |--cpu-features=FEATURES;CPU_FEATURES;comma-seperated list of CPU features;"

conf_test_cpu() {
    conf_depends_on tolower
    conf_depends_on target

    test -n "$CPU_FEATURES" && _cpu_features="$CPU_FEATURES"
    if test -n "$CPU" ; then
        _cpu="$CPU"
        echocheck "CPU model"
        echores "$_cpu"
        echocheck "CPU features"
        # FIXME: guess cpu features if not specified, but CPU is. helpful for
        # cross-compilation
        _cpu_features=`conf_guess_cpu_features $_cpu`
        echores "$_cpu_features"
        conf_mark_test_done cpu
        return
    fi

    conf_depends_on uname
    conf_depends_on os
    conf_depends_on arch

    _cpuinfo="echo"
    if test -r /proc/cpuinfo ; then
        _cpuinfo="cat /proc/cpuinfo"
    elif test -r /compat/linux/proc/cpuinfo ; then
        _cpuinfo="cat /compat/linux/proc/cpuinfo"
    elif test -r /usr/pkg/emul/linux/proc/cpuinfo ; then
        _cpuinfo="cat /usr/pkg/emul/linux/proc/cpuinfo"
    else
        conf_compile_tool_cpuinfo
    fi

    case "$_arch" in
        i386)       conf_cpu_i386 ;;
        x86_64)     conf_cpu_x86_64 ;;
        sparc*)     conf_cpu_sparc ;;
        alpha)      conf_cpu_alpha ;;
        powerpc*)   conf_cpu_powerpc ;;
        mips) ;;
    esac
    
    test -z "$_cpu" && _cpu=$_arch

    echocheck "CPU vendor"
    if test "$_cpu_family" = "unknown" ; then
        echores "$_cpu_vendor"
    else
        _res_comment="$_cpu_family:$_cpu_model:$_cpu_stepping"
        echores "$_cpu_vendor"
    fi
    echocheck "CPU model"
    _res_comment="$_cpu"
    echores "$_cpu_modelname"

    test -n "$CPU_FEATURES" && _cpu_features="$CPU_FEATURES"
    echocheck "CPU features"
    echores "$_cpu_features"

    conf_mark_test_done cpu
}

conf_compile_tool_cpuinfo() {
    conf_depends_on tempdir

    if test -z "$_tempdir" ; then
        return
    fi

    echocheckreal "compiling cpuinfo helper utility"

    case "$_arch" in
        x86_64)	 tmpa=i386 ;;
        sparc64) tmpa=sparc ;;
        *)       tmpa=$_arch ;;
    esac

    test -r "conf/tools/cpuinfo-$tmpa.c" || { \
        echores "not available for $_arch";   \
        return; }

    for i in "tcc" "$CC" "cc" "gcc" "icc" ; do
        { $i -o "$_tempdir"/cpuinfo-$tmpa-$$ conf/tools/cpuinfo-$tmpa.c ; } 2>/dev/null 1>&2
        if test $? -eq 0 ; then break ; fi
    done

    if test $? -eq 0 ; then
        _cpuinfo="$_tempdir"/cpuinfo-$tmpa-$$
        echores "done (using $_cpuinfo)"
    else
        echores "failed to compile"
    fi
}

conf_extract_cpuinfo() {
    {
        $_cpuinfo 2>/dev/null
        echo EOF
    } | {
        tmp=""
        read line
            IFS=$SAVEIFS
        while test "$line" != "EOF" ; do
            case "$line" in
                $1*) tmp=`echo "$line" | cut_fast ':' 2` ; break ;;
            esac
            IFS=$SAVEIFS
            read line
        done
        test -z "$tmp" && tmp=unknown
        echo $tmp
    }
}

conf_cpu_i386() {
    _cpu_modelname=`conf_extract_cpuinfo 'model name'`
    _cpu_vendor=`conf_extract_cpuinfo 'vendor'`
    _cpu_family=`conf_extract_cpuinfo 'cpu family'`
    _cpu_model=`conf_extract_cpuinfo 'model'`
    _cpu_stepping=`conf_extract_cpuinfo 'stepping'`
    _cpu_features=`conf_extract_cpuinfo 'features'`
    test "$_cpu_features" = "unknown" &&
                                   _cpu_features=`conf_extract_cpuinfo 'flags'`

    case "$_cpu_vendor" in
    AuthenticAMD|AMD\ IS\ BETTER|DEI|NexGenerationAMD)
        case "$_cpu_family" in
            3)  _cpu=i386 ;;
            4)  _cpu=i486 ;;
            5)  _cpu=i586
                case "$_cpu_model" in
                    0)  _cpu=i586 ;;
                    1)  _cpu=i586 ;;
                    2)  _cpu=i586 ;;
                    3)  _cpu=i586 ;;
                    6)  _cpu=k6 ;;
                    7)  _cpu=k6 ;;
                    8)  _cpu=k6-2 ;;
                    9)  _cpu=k6-3 ;;
                    13) _cpu=k6-3 ;;
                esac ;;
            6)  _cpu=athlon
                case "$_cpu_model" in
                    1)  _cpu=athlon ;;
                    2)  _cpu=athlon ;;
                    3)  _cpu=athlon ;;
                    4)  _cpu=athlon ;;
                    6)  _cpu=athlon-xp ;;
                    7)  _cpu=athlon-xp ;;
                    8)  _cpu=athlon-xp ;;
                    10) _cpu=athlon-xp ;;
                esac ;;
            15) _cpu=k8 ;;
        esac
        ;;
    GenuineIntel)
        case "$_cpu_family" in
            3)  _cpu=i386 ;;
            4)  _cpu=i486 ;;
            5)  _cpu=i586
                case "$_cpu_model" in
                    0)  _cpu=i586 ;;
                    1)  _cpu=i586 ;;
                    2)  _cpu=i586 ;;
                    3)  _cpu=i586 ;;
                    4)  _cpu=pentium-mmx ;;
                    7)  _cpu=i586 ;;
                    8)  _cpu=pentium-mmx ;;
                esac ;;
            6)  _cpu=i686
                case "$_cpu_model" in
                    0)  _cpu=i686 ;;
                    1)  _cpu=i686 ;;
                    3)  _cpu=pentium2 ;;
                    5)  _cpu=pentium2 ;;
                    6)  _cpu=pentium2 ;;
                    7)  _cpu=pentium3 ;;
                    8)  _cpu=pentium3 ;;
                    9)  _cpu=pentium-m ;;
                    10) _cpu=pentium3 ;;
                    11) _cpu=pentium3 ;;
                    13) _cpu=pentium-m ;;
                    14) _cpu=pentium-m ;;
                esac ;;
            15) _cpu=pentium4
                case "$_cpu_model" in
                    0) _cpu=pentium4 ;;
                    1) _cpu=pentium4 ;;
                    2) _cpu=pentium4 ;;
                    3) _cpu=prescott ;;
                    4) _cpu=prescott ;;
                    6) _cpu=prescott ;;
                esac ;;
        esac
        ;;
    CentaurHauls)
        _cpu=i486
        case "$_cpu_model" in
            4)  _cpu=winchip-c6 ;;
            8)  _cpu=winchip2 ;;
            9)  _cpu=c3 ;;
        esac
        ;;
    CyrixInstead)
        case "$_cpu_family" in
            4)  _cpu=i486
                case "$_cpu_model" in
                    9)  _cpu=i586 ;;
                esac ;;
            5)  _cpu=i586
                case "$_cpu_model" in
                    4)  _cpu=i586 ;;
                    2)  _cpu=i686 ;;
                esac ;;
            6)  _cpu=i686
                case "$_cpu_model" in
                    0)  _cpu=i686 ;;
                esac ;;
        esac
        ;;
    NexGenDriven)
        _cpu=i586
        case "$_cpu_family" in
            5) _cpu=i586
            case "$_cpu_model" in
                0) _cpu=i586 ;;
            esac ;;
        esac
        ;;
    RiseRiseRise)
        _cpu=i586
        case "$_cpu_family" in
            5) _cpu=i586
            case "$_cpu_model" in
                0) _cpu=k6 ;;
                2) _cpu=k6 ;;
            esac ;;
        esac
        ;;
    SiS\ SiS\ SiS)
        _cpu=i586
        case "$_cpu_family" in
            5) _cpu=i586
            case "$_cpu_model" in
                0) _cpu=pentium-mmx ;;
            esac ;;
        esac
        ;;
    GenuineTMx86|TransmetaCPU)
        _cpu=i586
        case "$_cpu_family" in
            5) _cpu=i586
            case "$_cpu_model" in
                4) _cpu=pentium-mmx ;;
            esac ;;
            6) _cpu=i686 ;;
        esac
        ;;
    Geode\ by\ NSC)
        _cpu=i586
        case "$_cpu_family" in
            5) _cpu=i586
            case "$_cpu_model" in
                4)  _cpu=k6 ;;
                5)  _cpu=k6-2 ;;
                10) _cpu=k6-3 ;;
            esac ;;
        esac
        ;;
    unknown|UMC\ UMC\ UMC|*)
        case "$_cpu_family" in
            3)  _cpu=i386 ;;
            4)  _cpu=i486 ;;
            5)  _cpu=i586 ;;
            6)  _cpu=i686 ;;
        esac
        ;;
    esac
}

conf_cpu_x86_64() {
    _cpu_modelname=`conf_extract_cpuinfo 'model name'`
    _cpu_vendor=`conf_extract_cpuinfo 'vendor'`
    _cpu_family=`conf_extract_cpuinfo 'cpu family'`
    _cpu_model=`conf_extract_cpuinfo 'model'`
    _cpu_stepping=`conf_extract_cpuinfo 'stepping'`
    _cpu_features=`conf_extract_cpuinfo 'features'`
    test "$_cpu_features" = "unknown" &&
                                   _cpu_features=`conf_extract_cpuinfo 'flags'`

    case "$_cpu_vendor" in
    AuthenticAMD|AMD\ IS\ BETTER|DEI|NexGenerationAMD|IT\'S\ HAMMER\ TIME)
        _cpu=k8 ;;
    GenuineIntel)
        _cpu=nocona ;;
    *)
        _cpu=k8 ;;
    esac
}

conf_cpu_sparc() {
    _cpu_modelname=`conf_extract_cpuinfo 'cpu'`
    _cpu_vendor=`conf_extract_cpuinfo 'vendor_id'`
    _cpu_features=`conf_extract_cpuinfo 'features'`
    test -z "$_cpu_features" && _cpu_features=none
    _cpu=`conf_extract_cpuinfo 'type'`
    test "$_cpu" = "unknown" && test -n "$_uname" && _cpu=`$_uname -m`

    case "$_cpu" in
        sun4)	_cpu=v7 ;;
        sun4c)  _cpu=v7 ;;
        sun4d)  _cpu=v8 ;;
        sun4m)  _cpu=v8 ;;
        sun4u)  _cpu=v9 ;;
        *)      _cpu=v7 ;;
    esac
}

conf_cpu_powerpc() {
    case "$_os" in
    linux)
        _cpu=`conf_extract_cpuinfo 'cpu'`
        ;;
    darwin)
        conf_depends_on hostinfo || return
        conf_depends_on sed || return
        _cpu_modelname=`$_hostinfo | $_sed '/Processor type/!d' | cut_fast ':' 2`
        _cpu=`echo $_cpu_modelname | cut_fast ' ' 3 | $_sed 's/ppc//'`
        conf_depends_on sysctl && \
        if test "`$_sysctl -n hw.vectorunit`" -eq 1 ||
            "`$_sysctl -n hw.optional.altivec 2>/dev/null`" -eq 1 ; then
            _cpu_features="altivec"
        fi
        ;;
    netbsd)
        conf_depends_on sysctl || return
        if test "`$_sysctl -n machdep.altivec`" -eq 1 ; then
            _cpu_features="altivec"
        fi
        ;;
    aix)
        conf_depends_on lsattr || return
        conf_depends_on sed || return
        _cpu=`$_lsattr -E -l proc0 -a type | $_sed '/type/!d' | cut_fast ' ' 2 | $_sed 's/PowerPC_//'`
        ;;
    esac

    _cpu=`echo "$_cpu" | cut_fast ' ' 1 | $_tolower`

    # FIXME: somebody knowledgable about the ppc architecture should add
    # all known cpu's and extend the altivec table
    case "$_cpu" in
        603ev)              _cpu=603e ;;
        604e|604r|604ev)    _cpu=604 ;;
        740?750|745?755)    _cpu=740 ;;
        750cx)              _cpu=750 ;;
        970*)               _cpu=970 ;;
        unknown)            _cpu=$_arch ;;
    esac

    if test -z "$_cpu_features" ; then
        case "$_cpu" in
            7400|7450|g4|970|g5|power5)
                _cpu_features=altivec ;;
        esac
    fi

    _cpu_modelname="$_cpu"
}

conf_cpu_alpha() {
    _cpu_modelname=`conf_extract_cpuinfo 'cpu'`
    _cpu=`conf_extract_cpuinfo 'cpu model' | $_tolower`
    _cpu_vendor="DEC"
}

# Called in case CPU is overriden on the commandline (CPU=... or --target)
# FIXME: complete list
conf_guess_cpu_features() {
    case $1 in
        k8)         echo "mtrr mmx mmxext 3dnow 3dnowext sse sse2" ;;
        pentium3)   echo "mtrr mmx sse" ;;
        athlon-xp)  echo "mtrr mmx mmxext 3dnow 3dnowext sse" ;;
        power5)     echo "altivec" ;;
        *)          echo "none" ;;
    esac
}
