conf_module cxx

_cxx=
_cxx_vendor=
_cxx_vendor_long=
_cxx_version=
_cxx_version_major=
_cxx_version_minor=
_cxx_version_subminor=

conf_add_options "expert" \
    "--cxx=CXX;CXX;specify C++ compiler;"

# There is "some" code duplication between this test and the cc test, but
# I chose not to combine them, because if more (different) compilers are added,
# this approach is cleaner.

conf_test_cxx() {
    conf_depends_on program _cxx "C++ compiler" "$CXX CC c++ g++ icc" \
                                                        conf_test_cxx_validate
    conf_mark_test_done cxx
    test -n "$_cxx" || return

    echocheck "C++ compiler vendor"
    conf_cxx_vendor
    if test -z "$_cxx_vendor_long" ; then
        echores "$_cxx_vendor"
    else
        _res_comment="$_cxx_vendor"
        echores "$_cxx_vendor_long"
    fi

    conf_cxx_version

    test -n "$_cxx"
}

conf_test_cxx_validate() {
    $1 -v && return
    $1 -version && return
    $1 --version && return
    $1 -flags && return
}

conf_cxx_vendor() {
    tmp=`$_cxx --help 2>&1`
    tmp=`echo $tmp`
    case "$tmp" in
        Intel*)
            _cxx_vendor=icc
            _cxx_vendor_long="Intel(R) C/C++ Compiler"
            return
            ;;
    esac

    tmp=`$_cxx --version 2>&1`
    case "$tmp" in
        g++*|*GCC*)
            _cxx_vendor=gnu
            _cxx_vendor_long="GNU C++ Compiler"
            return
            ;;
    esac

    tmp=`$_cxx -flags 2>&1`
    case "$tmp" in
        -#*-###*|_____*)
            _cxx_vendor=suncc
            _cxx_vendor_long="SUNWspro C++ Compiler"
            return
            ;;
    esac

    _cxx_vendor=unknown
}

conf_cxx_version() {
    case "$_cxx_vendor" in
        icc)
            echocheck "C++ compiler version"
            _cxx_version=`$_cxx -v 2>&1 | cut_fast ' ' 2`
            ;;
        gnu)
            echocheck "C++ compiler version"
            _cxx_version=`$_cxx -dumpversion 2>&1`
            ;;
        suncc)
            conf_compile_tool_cxx_version
            ;;
        unknown)
            ;;
    esac
    if test "$_cxx_vendor" != "unknown" ; then
        _cxx_version_major=`echo $_cxx_version | cut_fast '.' 1`
        _cxx_version_minor=`echo $_cxx_version | cut_fast '.' 2`
        _cxx_version_subminor=`echo $_cxx_version | cut_fast '.' 3`
        echores "$_cxx_version"
    fi
}

conf_compile_tool_cxx_version() {
    conf_depends_on tempdir

    if test -z "$_tempdir" ; then
        echocheck "C++ compiler version"
        return
    fi

    echocheckreal "compiling cxx-version helper utility"

    test -r "conf/tools/cxx-version-$_cxx_vendor.c" || {
        echores "not available for $_cxx_vendor"
        return; }

    { $_cxx -o "$_tempdir"/cxx-version-$_cxx_vendor conf/tools/cxx-version-$_cxx_vendor.c ; } 2>/dev/null 1>&2

    if test $? -eq 0 ; then
        _res_comment="using $_tempdir/cxx-version-$_cxx_vendor"
        echores "done"
        echocheck "C++ compiler version"
        _cxx_version=`$_tempdir/cxx-version-$_cxx_vendor 2>/dev/null`
    else
        echores "failed to compile"
        echocheck "C++ compiler version"
    fi
}
