/* cpuinfo utility - based on original cpuinfo from MPlayer, cpudetect from
 * MPlayer and cputest from libavcodec.
 *
 * Original cpuinfo.c (C) Various authors, MPlayer team
 * cpudetect.c (C) Various authors, MPlayer team
 * cputest.c (C)1997-1999 H. Dietz and R. Fisher. Converted to C and improved
 * by Favrice Bellard
 *
 * (C)2006 Ivo van Poorten, mixture of various sources, making it small and
 * hopefully as portable as possible (old cpuinfo.c did not compile with
 * tcc and the resulting binary of icc 8.1 did not work correctly)
 */

#include <stdio.h>

#ifdef ARCH_X86_64
#  define REG_b "rbx"
#  define REG_S "rsi"
#else
#  define REG_b "ebx"
#  define REG_S "esi"
#endif

#if defined(__INTEL_COMPILER)
#pragma warning(disable:810)
#pragma warning(disable:1418)
#endif

/* Where's some good documentation on the asm syntax of sunpro_c? This is
 * pretty much an ugly hack, but it works on Solaris x86 without gcc */
#if defined(__SUNPRO_C) || defined(__SUN_PRO_CC)
unsigned int index, vareax, varebx, varecx, varedx;
#define cpuid(INDEX,eax,ebx,ecx,edx)	        \
    index=INDEX;                                \
    asm("movl %ebx, %esi;   movl index, %eax;   \
         cpuid;	                                \
         movl %eax, vareax; movl %ebx, varebx;  \
         movl %ecx, varecx; movl %edx, varedx;  \
         xchg %ebx, %esi");                     \
    eax=vareax; ebx=varebx; ecx=varecx; edx=varedx;
#else
#define cpuid(index,eax,ebx,ecx,edx)    \
    __asm __volatile                    \
    ("mov %%"REG_b", %%"REG_S"\n\t"     \
     "cpuid\n\t"                        \
     "xchg %%"REG_b", %%"REG_S          \
     : "=a" (eax), "=S" (ebx),          \
       "=c" (ecx), "=d" (edx)           \
     : "0" (index));
#endif

#define store32(dst, src)               \
    (dst)[0] = (char) (src>> 0) & 0xff; \
    (dst)[1] = (char) (src>> 8) & 0xff; \
    (dst)[2] = (char) (src>>16) & 0xff; \
    (dst)[3] = (char) (src>>24) & 0xff;

#define stepping(x)     ((x>> 0) & 0x0f)
#define model(x)        ((x>> 4) & 0x0f)
#define family(x)       ((x>> 8) & 0x0f)
#define type(x)         ((x>>12) & 0x03)
#define extmodel(x)     ((x>>16) & 0x0f)
#define extfamily(x)    ((x>>20) & 0xff)

#define brandID(x)      ((x>> 0) & 0xff)
#define CLFLUSH(x)      ((x>> 8) & 0xff)
#define CPUcount(x)     ((x>>16) & 0xff)
#define APICID(x)       ((x>>24) & 0xff)

typedef struct bitfields_s {
    int   bit;
    char *shortname;
    char *longname;
} bitfields_t;

static bitfields_t STDFLAGS1[] = {
    { 0, "fpu" }, { 1, "vme"  }, { 2, "de"  }, { 3, "pse" }, { 4, "tsc" },
    { 5, "msr" }, { 6, "pae"  }, { 7, "mce" }, { 8, "cx8" }, { 9, "apic"},
    {11, "sep" }, {12, "mtrr" }, {13, "pge" }, {14, "mca" }, {15, "cmov"},
    {16, "pat" }, {17, "pse36"}, {18, "psn" }, {19, "clfl"}, {21, "dtes"},
    {22, "acpi"}, {23, "mmx"  }, {24, "fxsr"}, {25, "sse" }, {26, "sse2"},
    {27, "ss"  }, {28, "htt"  }, {29, "tm"  }, {30, "ia64"}, {31, "pbe" },
    {-1, ""    }
};

static bitfields_t STDFLAGS2[] = {
    { 0, "sse3"}, { 3, "monitor"}, { 4, "ds-cpl"}, { 5, "vmx"  }, { 7, "est"},
    { 8, "tm2" }, {10, "cid"    }, {13, "cx16"  }, {14, "etprd"}, {-1, ""   }
};

static bitfields_t AMDFLAGS1[] = {
    {11, "syscall" }, {19, "mp"    }, {20, "nx"      }, {22, "mmxext"},
    {25, "fxsr_opt"}, {27, "rdtscp"}, {30, "3dnowext"}, {31, "3dnow" },
    {-1, ""        }
};

static bitfields_t AMDFLAGS2[] = {
    { 0, "lahf_lm"}, { 1, "cmp_legacy"}, { 2, "svm"}, { 4, "cr8"}, {-1, ""}
};

unsigned int stdflags1 = 0, stdflags2 = 0, amdflags1 = 0, amdflags2 = 0;

int main(int argc, char **argv) {
    unsigned int eax, ebx, ecx, edx, i, j, intelcpu = 0;
    unsigned int max_std_level = 0, max_ext_level = 0;
    char vendorid[13], modelname[49];
    static unsigned int *flags[] = { &stdflags1, &stdflags2,
                                     &amdflags1, &amdflags2  };
    static bitfields_t  *masks[] = {  STDFLAGS1,  STDFLAGS2,
                                      AMDFLAGS1,  AMDFLAGS2  };

    cpuid(0, max_std_level, ebx, ecx, edx);
    store32(vendorid  , ebx);
    store32(vendorid+4, edx);
    store32(vendorid+8, ecx);
    vendorid[12]='\0';
    printf("vendor_id\t: %s\n", vendorid);
    if (edx == 0x49656e69 && ecx == 0x6c65746e) intelcpu = 1;

    if (max_std_level >= 1) {
        cpuid(1, eax, ebx, stdflags2, stdflags1);
        printf("cpu family\t: %u\n",    family(eax));
        printf("model\t\t: %u\n",       model(eax));
        printf("stepping\t: %u\n",      stepping(eax));
    }

    cpuid(0x80000000, max_ext_level, ebx, ecx, edx);
    if (max_ext_level >= 1) {
        cpuid(0x80000001, eax, ebx, amdflags2, amdflags1);
    }
    if (max_ext_level >= 4) {
        for (i=0; i<=2; i++) {
            cpuid(0x80000002+i, eax, ebx, ecx, edx);
            store32(modelname + i*16     , eax);
            store32(modelname + i*16 +  4, ebx);
            store32(modelname + i*16 +  8, ecx);
            store32(modelname + i*16 + 12, edx);
        }
        modelname[48]='\0';
        if (!intelcpu) printf("model name\t: %s\n", modelname);
        else printf("model name\t: Intel CPU\n");
    }

    printf("flags\t\t:");
    for (i=0; i<4; i++) {
        for (j=0; masks[i][j].bit >= 0; j++) {
            if (*(flags[i]) & (1 << masks[i][j].bit)) {
                printf(" %s", masks[i][j].shortname);
            }
        }
    }
    printf("\n");

    return 0;
}
